% Testing the NN distance distribution
function NN_distances()
    close all;
    clear all;

    generateFiles = false;%true;
    systematicSampling = true;
    % random walk - type = 0
    % uniform     - type = 1
    % normal      - type = 2
    % lightcurves - type = 3

    for type = 2:2
        if (type == 0)
            dir = 'randomwalk1/';
        elseif (type == 1)
            dir = 'uniform/';
        elseif (type == 2)
            dir = 'normal/';
        else
            dir = 'lightcurves/'
        end;
        fileName = ['../data/' dir 'TS0.txt'];
        outFile1 = ['../data/' dir 'NN_dists_all.txt'];
        outFile2 = ['../data/' dir 'NN_dists_sample02.txt'];    
    
        if (generateFiles)
        % compute NN distances the data set
             DB = load(fileName);    
%             sampleSize = 1; % the entire data set
%             disp(['Sample size = ' num2str(sampleSize) '--------------------']);
%             NN_DB_inds = computeNNDists(DB, sampleSize, systematicSampling);
%             save(outFile1, 'NN_DB_inds', '-ascii');
%             plotDistribution(NN_DB_inds, type)    
            sampleSize = 0.1; % 10 percent of the data
            disp(['Sample size = ' num2str(sampleSize) '--------------------']);    
            NN_sample_inds = computeNNDists(DB, sampleSize, systematicSampling);
            save(outFile2, 'NN_sample_inds', '-ascii');
            plotDistribution(NN_sample_inds, type);
        else
            NN_DB_inds = load(outFile1);
            NN_sample_inds = load(outFile2);
            plotFittedDistribution(NN_DB_inds, NN_sample_inds, type);
        end;
    end;

   
function D = computeNNDists(DB, sampleSize, systematicSampling)
    if (systematicSampling && sampleSize ~= 1) 
        [DB_sample, inds] = getSystematicSample(DB, sampleSize);
    else 
        [DB_sample, inds] = getSample(DB, sampleSize);
    end;
    D = zeros(size(DB_sample,1),2);
    for i = 1:size(DB_sample,1)
        if (rem(i, 10) == 0)
            disp(['Computing NN to element#' num2str(i) ' out of ' num2str(size(DB_sample,1))]);
        end
        nnD = nnDist(DB_sample(i,:), i, DB_sample);
        D(i,1) = inds(i,1);
        D(i,2) = nnD;
    end;
    
function [S, inds] = getSystematicSample(DB, sampleSize)  
    [S, inds] = getSample(DB, sampleSize/2);
    S1 = S;
    for i = 1:size(S1,1)
        if (rem(i, 10) == 0)
            disp(['Adding local samples#' num2str(i) ' of ' num2str(size(S1,1))]);
        end
        [D, ind] = nnDist(S1(i,:), inds(i,1), DB);
        existInds = find(inds == ind);
        if (size(existInds,1) == 0)
            S = [S; DB(ind,:)];
            inds = [inds; ind];
        end;
    end;
    disp(['Exact sample size=' num2str(size(S,1))]);
    inds = sort(inds,1);
    S = DB(inds,:);
    
function [S, inds] = getSample(DB, sampleSize)
    s_size = round(size(DB,1)*sampleSize);
    p = randperm(size(DB,1));
    pp = p(1,1:s_size);
    pp = sort(pp,2);
    inds = pp';
    S = DB(pp, :);
    
function plotDistribution(NN_sample_inds, type)
    data = NN_sample_inds(:,2);
    if (type == 0) % random walk
        x = 0:1:25;
    elseif (type == 1) % uniformly distributed
        x = 0:0.005:0.1;
    elseif (type == 2) % normally distributed
        x = 0:0.01:0.5;
    else
        x = 0:1:25;    % lightcurves
    end;
    n = hist(data, x)
    figure;
    plot(x, n, 'ob', 'LineWidth', 2);
    axis tight;
    
function plotFittedDistribution(NN_DB_inds, NN_sample_inds, type) 
    figure;
    dataDB = NN_DB_inds(:,2);
    data = NN_sample_inds(:,2);
    if (type == 0) % random walk
        x = 0:1:25;
    elseif (type == 1) % uniformly distributed
        x = 0:0.002:0.07;
    elseif (type == 2) % normally distributed
        x = 0:0.01:0.2;
    else
        x = 0:0.5:20;  % lightcurves 
    end;
    nDB = hist(dataDB, x);
    nDB = nDB/size(dataDB,1);
    n = hist(data, x);
    n = n/size(data,1);
    pDB = polyfit(x, nDB, 6);
    p = polyfit(x, n, 6);
    if (type == 0) % random walk
        x1 = 3:0.1:25;
    elseif (type == 1) % uniformly distributed
        x1 = 0:0.001:0.07;
    elseif (type == 2) % normally distributed
        x1 = 0:0.001:0.2;
    else
        x1 = 0:0.01:20; %lightcurves
    end;
    pDB_val = polyval(pDB, x1);
    p_val = polyval(p, x1);
    
    h1 = plot(x, nDB, 'ob', 'LineWidth', 2);
    hold on;
    plot(x1, pDB_val, 'b', 'LineWidth', 2); 
  %  h2 = plot(x, n, '*r', 'LineWidth', 2);
  %  plot(x1, p_val, 'r', 'LineWidth', 1);
 %   legend([h1, h2], '10000 elems', '1000 elems');
    axis tight;
    axis([2 25 0 0.115]);
    if (type == 0)
        title('NN distance distribution (random walk)')
    elseif (type == 1)
        title('NN distance distribution (uniform)')
    elseif (type == 2)
        title('NN distance distribution (normal)')
    else
        title('NN distance distribution (lightcurves)');
    end;
    

function plotFittedDistribution_1(NN_DB_inds, NN_sample_inds, type) 
    figure;
    dataDB = NN_DB_inds(:,2);
    data = NN_sample_inds(:,2);
    if (type == 0) % random walk
        x = 0:1:25;
        x1 = 0:0.1:25;
    elseif (type == 1) % uniformly distributed
        x = 0:0.002:0.07;
    else               % normally distributed
        x = 0:0.01:0.2;
    end;
    nDB = hist(dataDB, x);
    nDB = nDB/size(dataDB,1);
    n = hist(data, x);
    n = n/size(data,1);
    
    % compute MLE for the means 
    mDB = meanLogNorm(nDB');
    mSample = meanLogNorm(n);
    % compute MLE for the variances
    sDB = stdLogNorm(nDB, mDB);
    sSample = stdLogNorm(n, mSample);
    % compute the log-normal distribution
    f_DB = logNorm(x1, mDB, sDB);
    
    plot(x1, f_DB, 'b', 'LineWidth', 1);
    hold on;
    plot(x, nDB, 'ob', 'LineWidth', 2);
    %plot(x, n, 'or', 'LineWidth', 2);
    
function m = meanLogNorm(data) 
    l = log(data);
    l(find(l == -Inf),1) = 0;
    m = sum(l)/size(data,1);
    
function s = stdLogNorm(data, m)    
    s = sum((log(data)-m).^2)/size(data,1);
    
function f = logNorm(x, m, s)
    sq_lnx_m = (log(x)-m).^2;
    sq_s = 2*(s^2);
    x_s = x*s*(sqrt(2*pi));
    f = exp(-sq_lnx_m/sq_s)./x_s;