function dataMgrRandomWalk()
    close all;
    clear all;
    
    dir = '../data/randomwalk_1mill/'; % I/O directory
    ts_size = 512;                % length of each time series in the data base  
    ts_num = 10000;                % number of time series in each db page  
    db_pages = 100;               % number of db pages  
    DC = loadDiscords(dir, ts_size);     % load a number of discords to be planted among the random walks
    plotDiscords(DC);
    DC_pos = [10 50 70 80 90 100];    % page numbers where each discords will be placed
%    DC_pos = [1 5 7 8 9 10];    % page numbers where each discords will be placed
        
%    createRandomWalkData(dir, DC, DC_pos, ts_size, ts_num, db_pages);
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% Generate 1000 files with 1000 time series each. Every time series is a z-normalized random walk    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function createRandomWalkData(dir, DC, DC_pos, ts_size, ts_num, db_pages)
    formatspec = createFormatSpec(ts_size);
    rand('state', 1000);
    randn('state', 1000);
    disc_pos = 1;
    for files_num = 1:db_pages
        disp(['Creating file#' num2str(files_num)]);
        R_ts = zeros(ts_num, ts_size); 
        for i = 1:ts_num
            R_ts(i,:) = randomWalk(ts_size);
        end;
        if (files_num == DC_pos(1, disc_pos)) 
            % substitute the first random sequence with the corresponding
            % discord
            R_ts(1,:) = DC(disc_pos,:);
            if (disc_pos < size(DC_pos,2))
                disc_pos = disc_pos + 1;
            end;
        end;
        storeSeries(dir, R_ts, files_num-1, db_pages, formatspec);
    end;
    
function FS = createFormatSpec(ts_size)
    FS = '';
    for i = 1:ts_size
        FS = [FS '%-6.4f\t'];
    end;
    
function storeSeries(dir, TimeSeries, num, db_pages, formatspec)
    fileName = [dir 'TS'];
    fileName = [dir 'TS'  num2str(num) '.txt'];
    %save(fileName, 'TimeSeries', '-ascii');
    fid = fopen(fileName,'w');
    fprintf(fid,[formatspec '\n'],TimeSeries');
    fclose(fid);    

function y = randomWalk(ts_size)
    % Create a Random Walk y(t)=y(t-1)+e(t)
    % where e is a N(0,1)
    starty=50*rand(1); % a uniformly distributed random first point
    y=zeros(1,ts_size);
    e=randn(1,ts_size);
    y(1) = starty;
    for i=1:(ts_size-1),
        y(i+1)=y(i)+e(i+1);
    end
    % z normalize
    y = (y - mean(y,2))/std(y, 0, 2);
    time=[1:1:ts_size];
    
    %figure;
    %plot(time,y, 'b', 'LineWidth', 2);
    
    
function DC = loadDiscords(dir, ts_size)
    fileName = [dir 'discords.txt'];
    DCin = load(fileName);
    DC = [];
    for i = 1:size(DCin,1)
        ts = DCin(i,:);
        if (size(ts, 2) ~= ts_size) 
            ts = resample(ts, ts_size);
        end;
        ts = (ts - mean(ts,2)) / std(ts, 0, 2);
        DC = [DC; ts];
    end;
        
function plotDiscords(D)
    subplot(3,2,1);
    plot(D(1, :), 'b', 'LineWidth', 2);
    axis tight;
    set(gca, 'FontSize', 16);    
    box off;

    subplot(3,2,2);
    plot(D(2, :), 'b', 'LineWidth', 2);
    axis tight;
    set(gca, 'FontSize', 16);    
    box off;
    
    subplot(3,2,3);
    plot(D(3, :), 'b', 'LineWidth', 2);
    axis tight;
    set(gca, 'FontSize', 16);    
    box off;
    
    subplot(3,2,4);
    plot(D(4, :), 'b', 'LineWidth', 2);
    axis tight;
    set(gca, 'FontSize', 16);    
    box off;
    
    subplot(3,2,5);
    plot(D(5, :), 'b', 'LineWidth', 2);
    axis tight;
    set(gca, 'FontSize', 16);    
    box off;
    
    subplot(3,2,6);
    plot(D(6, :), 'b', 'LineWidth', 2);
    axis tight;
    set(gca, 'FontSize', 16);    
    box off;
    