function discdiscords()
    close all;
    clear all;
    
    threshold = 10;  % the best discord is at least threshold dist away from its nearest neighbor
    dataDir = ['../data/lightcurves/'];
    inFile = [dataDir 'timeseries_9K_sorted.tts'];   % time series data base
    k = 10;  % detect the first k discords
    
    tic
    [DB, Lbls] = loadDB(dataDir, inFile);
    %saveAsPages(dataDir, DB, Lbls);
    % threshold = computeThreshold(DB)
    [MM, indxPtr, MMvolume] = buildMainMemory(DB, threshold);   
    figure;
    MMvolume = MMvolume ./ size(DB,1);
    plot(MMvolume(:,1), 'b', 'LineWidth', 2);
    title('Memory Size');
    if (size(MM,1) < k) 
        k = size(MM,1);
    end;
    [discordMMInds, nnDists] = findDiscords1(DB, MM, indxPtr, k);
    toc
    indxPtr(discordMMInds)
    plotDiscords(DB(indxPtr(discordMMInds),:), Lbls(indxPtr(discordMMInds)), nnDists);
    checkPrecisionRecall(MM, indxPtr, DB, threshold, Lbls);
    
function checkPrecisionRecall(MM, indxPtr, DB, threshold, Lbls) 
    [MM, indxPtrMem] = cleanMainMemory(DB, threshold, MM, indxPtr);
    num_anomalies = 0;
    for i = 1:size(Lbls,1)
        str = cell2mat(cell(Lbls(i)));    
        k = findstr('ANOM', str);
        if (size(k,1) ~= 0)
            num_anomalies = num_anomalies + 1;
        end;
    end;
    disp(['----------------------------------Precision-Recall(threshold=10)--------------------------']);
    disp(['Total number of anomalies: 60']);
    disp(['Detected anomalies: ' num2str(num_anomalies)]);
    disp(['Total # of true discords ' num2str(size(MM,1))]);
    disp(['Recall = ' num2str(num_anomalies / 60)]);
    disp(['Precision = ' num2str(num_anomalies/size(MM,1))]);
    disp(['----------------------------------------------------------------------------']);    
    
function saveAsPages(dir, DB, Lbls) 
    fileName = [dir 'TS0000.txt'];
    formatspec = createFormatSpec(1024);
    % save time series
     fid = fopen(fileName,'w');
     fprintf(fid,[formatspec '\n'],DB');
     fclose(fid);  
    % save labels
    fileName = [dir 'TS0000_labels.txt'];
    formatspec = '%s\n';
    fid = fopen(fileName,'w');
    for i = 1:size(Lbls,1)
        str = cell2mat(cell(Lbls(i)));
        fprintf(fid, formatspec, str');
    end;
    fclose(fid);
    
function FS = createFormatSpec(ts_size)
    FS = '';
    for i = 1:ts_size
        FS = [FS '%-6.4f\t'];
    end;

% find discords using only early abandoning    
function [discordMMInds, nnDists] = findDiscords1(DB, MM, indxPtr, k)
    discordInds = [];
    nnDists = [];
    earlyAbandon = true;
    znormalize = false;
    for i = 1:size(MM, 1)
        disp(['Discord candidate ' num2str(i) ' out of ' num2str(size(MM,1))]);
        tmpMaxMinNNDist = nnDist(MM(i,:), indxPtr(i), DB);
        %Res = mex_nndist(MM(i,:), DB, indxPtr(i)-1, earlyAbandon, znormalize); %cpp code
        %tmpMaxMinNNDist = Res(3,1);
        
        if (size(discordInds,1) == 0)
            discordInds = [i]; 
            nnDists = [tmpMaxMinNNDist];
        else
            pos = 1;      
            lk = min(k, size(nnDists,1));
            while (pos <= lk & nnDists(pos) > tmpMaxMinNNDist)
                pos = pos + 1;
            end;
            if (pos > 1 & pos < lk)
                discordInds = [discordInds(1:pos-1,:);i;discordInds(pos:lk,:)];
                nnDists = [nnDists(1:pos-1,:);tmpMaxMinNNDist;nnDists(pos:lk,:)];                
            elseif (pos == 1)
                discordInds = [i;discordInds(1:lk,:)];
                nnDists = [tmpMaxMinNNDist;nnDists(1:lk,:)];                
            elseif (pos <= k)
                discordInds = [discordInds(1:lk,:);i];
                nnDists = [nnDists(1:lk,:);tmpMaxMinNNDist];                
            end;
        end;
        if (size(discordInds,1) > k)
            discordInds = discordInds(1:k);
            nnDists = nnDists(1:k);
        end;
    end;
    %M = [discordInds nnDists]
    discordMMInds = discordInds(1:k,:);
    nnDists = nnDists(1:k,:);
    
function X = swap(oldX, pos1, pos2)
    X = oldX;
    tmp = X(pos1,:);
    X(pos1,:) = X(pos2,:);
    X(pos2,:) = tmp;
    
function [DB, Lbls] = loadDB(dataDir, inFile)
    inFile1 = '';
    isomap_util = cluster.IsomapUtil;
    v = isomap_util.loadData(inFile);  
    DB = v.get(0);
    Lbls = v.get(1);
    
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Plotting routines
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plotDiscords(discords, lbls, dists)
    for i = 1:size(discords,1)
        figure;
        plot(discords(i,:), 'b', 'LineWidth', 3);
        set(gca, 'FontSize', 14);
        str = cell2mat(cell(lbls(i)));
        title(['Discord' num2str(i) ' ' str ' NNDist=' num2str(dists(i))]);
    end;
