function testRandomWalk()
    close all;
    clear all;
    
    %dir = '../data/randomwalk/';
    dir = '../data/lightcurves/';
    
    mem_pages = 10;    
    k = 10; % the top k discords
    
    fos = java.io.FileOutputStream([dir 'Output.txt']);
    % computing the right threshold
    startTime = java.lang.System.currentTimeMillis;
    %threshold = computeRWThreshold(dir);
    
    %threshold = 21.45;  %Use this threshold for random walk
    threshold = 10; %Use this threshold for light curves
    endTime = java.lang.System.currentTimeMillis;
    log_line(fos, ['Time to compute threshold: ' num2str((endTime - startTime)/1000) 'sec.']);
    log_line(fos, ['Best threshold(0.01%): ' num2str(threshold)]);
    
    % building the main memory
    tic
    startTime = java.lang.System.currentTimeMillis;
    [MM, indxPtr, MMvolume] = buildRWMainMemory(dir, threshold, mem_pages);
    endTime = java.lang.System.currentTimeMillis;
    save([dir 'Output_memvolume.txt'], 'MMvolume', '-ascii');
    log_line(fos, ['Time to build main memory: ' num2str((endTime-startTime)/1000) 'sec.']);    
    figure;
    plot(MMvolume, 'o-b', 'LineWidth', 2);
    title('Memory size (Phase 1)');
    
    % find the discords amont the elements in the main memory
    startTime = java.lang.System.currentTimeMillis;
    [MM, indxPtr, nnDists, MMvolume] = findRWDiscords(dir, threshold, mem_pages, k, MM, indxPtr);
    toc
    endTime = java.lang.System.currentTimeMillis;
    figure;
    plot(MMvolume, 'b', 'LineWidth',2);
    title('Memory size (Phase 2)');
    log_line(fos, ['Time to build find the discords: ' num2str((endTime-startTime)/1000) 'sec.']);    
    logDiscords(dir, MM, indxPtr, nnDists);
    fos.close;    
    
    % plot discords
    for i = 1:size(MM,1)
        figure;
        plot(MM(i,:), 'b', 'LineWidth', 2);
        title(['Discord #' num2str(i) ' Indx=' num2str(indxPtr(i)) ' NNDist=' num2str(nnDists(i))]);
    end;
    
function [MM, indxPtr, nnDists, MMvolume] = findRWDiscords(dir, threshold, mem_pages, k, MM, indxPtr)
    nnDists = realmax*ones(size(MM,1),1);
    offset = 0;
    MMvolume = [size(MM,1)];
    for i = 0:mem_pages-1
        disp(['---(Second phase) Checking DB page#' num2str(i+1) ' of ' num2str(mem_pages)]);
        DB = loadDB(dir, i, mem_pages);    
        [MM, indxPtr, nnDists ] = findDiscordNN(DB, MM, indxPtr, nnDists, threshold, offset);
        offset = offset + size(DB,1);
        MMvolume = [MMvolume; size(MM, 1)];
    end;
    [MM, indxPtr, nnDists] = findTopKDiscords(MM, indxPtr, nnDists, k);
        
function [MM, indxPtr, MMvolume] = buildRWMainMemory(dir, threshold, mem_pages)    
    MM = [];
    indxPtr = [];
    MMvolume = zeros(mem_pages,1);
    offset = 0;
    for i = 0:mem_pages-1
        disp(['---Building MainMemory: page#' num2str(i+1) ' of ' num2str(mem_pages) ' MMVolume=' ]);
        DB = loadDB(dir, i, mem_pages);
        for i = 1:5
            subplot(5,2, (i-1)*2 + 1);
            plot(DB((i-1)*2 + 1, :), 'r', 'LineWidth', 2);
            axis off;
            subplot(5,2, (i-1)*2 + 2);
            plot(DB((i-1)*2 + 2,:), 'r', 'LineWidth', 2);
            axis off;
        end;
        [MM, indxPtr, tmpMMvolume] = buildMainMemory(DB, threshold, MM, indxPtr, offset);
        offset = offset + size(DB,1);
        MMvolume(i+1,1) = size(MM,1);
    end;

function T = computeRWThreshold(dir)
    file = [dir 'TS0000.txt'];
    DB = load(file);
    samplePerc = 20;
    T = computeThreshold(DB, samplePerc);
    clear DB;
    
function DB = loadDB(dir, num, db_pages)
%     fileName = [dir 'TS'];
%     if (num == 0)
%         zero_nums = db_pages;
%     else
%         zero_nums = db_pages / num;
%     end;
%     suff = '';
%     counter = 1;
%     while (counter < zero_nums)
%         suff = [suff '0'];
%         counter = counter * 10;
%     end;
%     fileName = [fileName suff num2str(num) '.txt'];

    fileName = [dir 'TS' num2str(num) '.txt'];
    
    DB = load(fileName);
    
function logDiscords(dir, MM, indxPtr, nnDists)
    fileName = [dir 'Output_discords.txt'];
    save(fileName, 'MM', '-ascii');
    indx_dist = [indxPtr nnDists];
    fileName = [dir 'Output_indxdists.txt'];
    save(fileName, 'indx_dist', '-ascii');
    
function log_line(fos, str)
    javaStr = java.lang.String(str);
    fos.write(javaStr.getBytes);
    fos.write(10);
    fos.flush;
